/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "main.h"

/*
===============================================================================

				COMMON BUILT-IN FUNCTIONS

===============================================================================
*/

const char * PC_VarString (int first)
{
	int  i;
	static char out[256];
	
	out[0] = 0;

	for (i = first ; i < pr->argc ; i++)
	{
		strcat (out, G_STRING((OFS_PARM0+i*3)));
	}

	return out;
}


/*
=================
PC_error

This is a TERMINAL error, which will kill off the entire server.

error(value)
=================
*/
void PC_error (void)
{
	const char	*s;

	s = PC_VarString(0);

	Con_Printf ("======SERVER ERROR in %s:\n%s\n",
		 pr->strings + pr->xfunction->s_name, s);

	Host_Error ("Program error");
}


/*
=================
PC_normalize

vector normalize(vector)
=================
*/
void PC_normalize (void)
{
	float	*value1;
	vec3_t	newvalue;
	float	new;
	
	value1 = G_VECTOR(OFS_PARM0);

	new = value1[0] * value1[0] + value1[1] * value1[1] + value1[2]*value1[2];
	new = sqrt(new);
	
	if (new == 0)
		newvalue[0] = newvalue[1] = newvalue[2] = 0;
	else
	{
		new = 1/new;
		newvalue[0] = value1[0] * new;
		newvalue[1] = value1[1] * new;
		newvalue[2] = value1[2] * new;
	}
	
	VectorCopy (newvalue, G_VECTOR(OFS_RETURN));	
}

/*
=================
PC_vlen

scalar vlen(vector)
=================
*/
void PC_vlen (void)
{
	float	*value1;
	float	new;
	
	value1 = G_VECTOR(OFS_PARM0);

	new = value1[0] * value1[0] + value1[1] * value1[1] + value1[2]*value1[2];
	new = sqrt(new);
	
	G_FLOAT(OFS_RETURN) = new;
}

/*
=================
PC_vectoyaw

float vectoyaw(vector)
=================
*/
void PC_vectoyaw (void)
{
	float	*value1;
	float	yaw;
	
	value1 = G_VECTOR(OFS_PARM0);

	if (value1[1] == 0 && value1[0] == 0)
		yaw = 0;
	else
	{
		yaw = (int) (atan2(value1[1], value1[0]) * 180 / M_PI);
		if (yaw < 0)
			yaw += 360;
	}

	G_FLOAT(OFS_RETURN) = yaw;
}


/*
=================
PC_vectoangles

vector vectoangles(vector)
=================
*/
void PC_vectoangles (void)
{
	float	*value1;
	float	forward;
	float	yaw, pitch;
	
	value1 = G_VECTOR(OFS_PARM0);

	if (value1[1] == 0 && value1[0] == 0)
	{
		yaw = 0;
		if (value1[2] > 0)
			pitch = 90;
		else
			pitch = 270;
	}
	else
	{
		yaw = (int) (atan2(value1[1], value1[0]) * 180 / M_PI);
		if (yaw < 0)
			yaw += 360;

		forward = sqrt (value1[0]*value1[0] + value1[1]*value1[1]);
		pitch = (int) (atan2(value1[2], forward) * 180 / M_PI);
		if (pitch < 0)
			pitch += 360;
	}

	G_FLOAT(OFS_RETURN+0) = pitch;
	G_FLOAT(OFS_RETURN+1) = yaw;
	G_FLOAT(OFS_RETURN+2) = 0;
}


/*
=================
PC_Random

Returns a number from 0.0 to 1.0 (full range)

random()
=================
*/
void PC_random (void)
{
	G_FLOAT(OFS_RETURN) = (rand() & 0x7fff) / (float)0x7fff;
}


/*
=================
PC_break

break()
=================
*/
void PC_break (void)
{
	Con_Printf ("break statement\n");
	*(int *)-4 = 0;	// dump to debugger
//	PR_RunError ("break statement");
}


/*
=================
PC_cvar

float cvar (string)
=================
*/
void PC_cvar (void)
{
	char	*str;
	
	str = G_STRING(OFS_PARM0);
	
	G_FLOAT(OFS_RETURN) = Cvar_VariableValue (str);
}

/*
=================
PC_cvar_set

float cvar (string)
=================
*/
void PC_cvar_set (void)
{
	char	*var, *val;
	
	var = G_STRING(OFS_PARM0);
	val = G_STRING(OFS_PARM1);
	
	Cvar_Set (var, val);
}


/*
=========
PC_dprint
=========
*/
void PC_dprint (void)
{
	Con_DPrintf ("%s",PC_VarString(0));
}

void PC_fabs (void)
{
	float	v;
	v = G_FLOAT(OFS_PARM0);
	G_FLOAT(OFS_RETURN) = fabs(v);
}


// andrewj: temp string is now part of string buffer (after the empty string)

void PC_ftos (void)
{
	float	v;
	v = G_FLOAT(OFS_PARM0);
	
	if (v == (int)v)
		sprintf (pr->strings + 1, "%d",(int)v);
	else
		sprintf (pr->strings + 1, "%5.1f",v);
	
	G_INT(OFS_RETURN) = 1;
}

void PC_vtos (void)
{
	sprintf (pr->strings + 1, "'%5.1f %5.1f %5.1f'", G_VECTOR(OFS_PARM0)[0], G_VECTOR(OFS_PARM0)[1], G_VECTOR(OFS_PARM0)[2]);

	G_INT(OFS_RETURN) = 1;
}


void PC_coredump (void)
{
	ED_PrintEdicts ();
}

void PC_traceon (void)
{
	pr->trace = true;
}

void PC_traceoff (void)
{
	pr->trace = false;
}


void PC_rint (void)
{
	float	f;
	f = G_FLOAT(OFS_PARM0);
	if (f > 0)
		G_FLOAT(OFS_RETURN) = (int)(f + 0.5);
	else
		G_FLOAT(OFS_RETURN) = (int)(f - 0.5);
}

void PC_floor (void)
{
	G_FLOAT(OFS_RETURN) = floor(G_FLOAT(OFS_PARM0));
}

void PC_ceil (void)
{
	G_FLOAT(OFS_RETURN) = ceil(G_FLOAT(OFS_PARM0));
}

void PC_sin (void)
{
	G_FLOAT(OFS_RETURN) = sin(G_FLOAT(OFS_PARM0));
}

void PC_cos (void)
{
	G_FLOAT(OFS_RETURN) = cos(G_FLOAT(OFS_PARM0));
}

void PC_sqrt (void)
{
	G_FLOAT(OFS_RETURN) = sqrt(G_FLOAT(OFS_PARM0));
}


void PC_Fixme (void)
{
	PR_RunError ("unimplemented builtin");
}


//--- editor settings ---
// vi:ts=4:sw=4:noexpandtab
