/*
Copyright (C) 1996-1997 Id Software, Inc.
Copyright (C)      2012 Andrew Apted

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "main.h"

/*
===============================================================================

				USER INTERFACE BUILT-IN FUNCTIONS

===============================================================================
*/


// andrewj: a lot of the engine code assumes globalvars_t is from the
//          game progs.  Only this file is aware of the UI variant,
//          which we access here using some C preprocessor hackitude.

#define globalvars_t  ui_globalvars_t
#define entvars_t     ui_entvars_t

#undef PROGHEADER_CRC

#include "../ui/prog_api.h"

#undef globalvars_t
#undef entvars_t


prog_vm_t  pr_ui;

#define UI_GLOBAL_VAR(name)  ( ((ui_globalvars_t *) pr_ui.global_struct)->name )


void PU_blah_blah (void)
{
	const char *s = G_STRING(OFS_PARM0);

	fprintf(stderr, "%s !!\n", s);
}


void PU_draw_text (void)
{
	int x = (int) G_FLOAT(OFS_PARM0);
	int y = (int) G_FLOAT(OFS_PARM1);

	const char *s = G_STRING(OFS_PARM2);

	Draw_String(x, y, s);
}


void PU_draw_bigtext (void)
{
	int x = (int) G_FLOAT(OFS_PARM0);
	int y = (int) G_FLOAT(OFS_PARM1);

	const char *s = G_STRING(OFS_PARM2);

	int col = (int) G_FLOAT(OFS_PARM3);

	Draw_BigString(x, y, s, col);
}


void PU_draw_image (void)
{
	int x = (int) G_FLOAT(OFS_PARM0);
	int y = (int) G_FLOAT(OFS_PARM1);

	const char *s = G_STRING(OFS_PARM2);

	V_DrawCacheImage(x, y, s);
}


void PU_draw_tile (void)
{
	int x = (int) G_FLOAT(OFS_PARM0);
	int y = (int) G_FLOAT(OFS_PARM1);

	const char *s = G_STRING(OFS_PARM2);

	int tw = (int) G_FLOAT(OFS_PARM3);
	int th = (int) G_FLOAT(OFS_PARM4);

	int tx = (int) G_FLOAT(OFS_PARM5);
	int ty = (int) G_FLOAT(OFS_PARM6);

	V_DrawCacheTile(x, y, s, tw, th, tx, ty);
}


void PU_play_sound (void)
{
	const char *s = G_STRING(OFS_PARM0);

	int sound_num = S_FindSound(s);

	S_StartFX(sound_num, CAT_UI, 1.0f);
}


void PU_play_music (void)
{
	const char *s = G_STRING(OFS_PARM0);

	S_OGG_Open(s);
}


void PU_leave_menu (void)
{
	key_dest = key_game;
}


void PU_localcmd (void)
{
	const char *s = G_STRING(OFS_PARM0);

	Cbuf_AddText(s);
}


// TEMP SHITE
#define GAME_W  1024
#define GAME_H  1024

static int * game_world;

void PU_gw_get (void)
{
	int x = (int) G_FLOAT(OFS_PARM0);
	int y = (int) G_FLOAT(OFS_PARM1);

	if (x < 0 || x >= GAME_W || y < 0 || y >= GAME_H)
	{
		G_FLOAT(OFS_RETURN) = 0;
		return;
	}

	G_FLOAT(OFS_RETURN) = game_world[y * GAME_W + x];
}

void PU_gw_put (void)
{
	int x = (int) G_FLOAT(OFS_PARM0);
	int y = (int) G_FLOAT(OFS_PARM1);
	int z = (int) G_FLOAT(OFS_PARM2);

	if (x < 0 || x >= GAME_W || y < 0 || y >= GAME_H)
		return;

	game_world[y * GAME_W + x] = z;
}


//============================================================================

builtin_t  pr_ui_builtins[] =
{
	// very first one is a dummy -- never used
	{ "<DUMMY>",  PC_Fixme },

	//======= COMMON =========

	{ "error",          PC_error },	// void(string e) error				= #10;

	{ "random",         PC_random },	// float() random						= #7;
	{ "rint",           PC_rint },
	{ "floor",          PC_floor },
	{ "ceil",           PC_ceil },
	{ "fabs",           PC_fabs },
	{ "sin",            PC_sin },
	{ "cos",            PC_cos },
	{ "sqrt",           PC_sqrt },
	{ "normalize",      PC_normalize },	// vector(vector v) normalize			= #9;
	{ "vlen",           PC_vlen },	// float(vector v) vlen				= #12;
	{ "vectoyaw",       PC_vectoyaw },	// float(vector v) vectoyaw		= #13;
	{ "vectoangles",    PC_vectoangles },

	{ "ftos",           PC_ftos },	// void(string s) ftos				= #26;
	{ "vtos",           PC_vtos },	// void(string s) vtos				= #27;

	{ "break",          PC_break },	// void() break						= #6;
	{ "coredump",       PC_coredump },
	{ "dprint",         PC_dprint },	// void(string s) dprint				= #25;
	{ "traceon",        PC_traceon },
	{ "traceoff",       PC_traceoff },

	{ "cvar",           PC_cvar },
	{ "cvar_set",       PC_cvar_set },

	//======== UI ===========

	{ "draw_text",      PU_draw_text },
	{ "draw_bigtext",   PU_draw_bigtext },
	{ "draw_image",     PU_draw_image },
	{ "draw_tile",      PU_draw_tile },

	{ "play_sound",     PU_play_sound },
	{ "play_music",     PU_play_music },

	{ "leave_menu",     PU_leave_menu },
	{ "localcmd",       PU_localcmd },

	// TEMP SHITE
	{ "gw_get",			PU_gw_get },
	{ "gw_put",			PU_gw_put },

	{ "blah_blah",      PU_blah_blah },  // for testing

	// end of list
	{ NULL, NULL }
};


//============================================================================


void UI_Init(void)
{
	game_world = Hunk_AllocName(sizeof(int) * GAME_W * GAME_H, "gworld");
}


void PR_ExecuteUIByName(const char *name)
{
	// grab the local time [FIXME: do this in Host_Frame, I guess]
	UI_GLOBAL_VAR(time) = host_time;

	prog_vm_t * save_pr = pr;

	pr = &pr_ui;

	PR_ExecuteProgramByName(name);

	pr = save_pr;
}


int UI_GetMenuState(void)
{
	if (! pr_ui.progs)
		return 0;

	return (int) UI_GLOBAL_VAR(m_state);
}


void UI_SetMouseState(void)
{
	if (! pr_ui.progs)
		return;
	
	UI_GLOBAL_VAR(mouse_x) = mouse_x;
	UI_GLOBAL_VAR(mouse_y) = mouse_y;
}


//--- editor settings ---
// vi:ts=4:sw=4:noexpandtab
