/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "pr_comp.h"			// defs shared with qcc
#include "../game/prog_api.h"	// generated by qcc

typedef union eval_s
{
	string_t		string;
	float			_float;
	float			vector[3];
	func_t			function;
	int				_int;
	int				edict;
} eval_t;	

#define	MAX_ENT_LEAFS	16
typedef struct edict_s
{
	bool_t	free;
	link_t		area;				// linked to a division node or leaf
	
	int			num_leafs;
	short		leafnums[MAX_ENT_LEAFS];

	entity_baseline_t	baseline;
	
	float		freetime;			// sv.time when the object was freed
	entvars_t	v;					// C exported fields from progs
// other fields from progs come immediately after
} edict_t;
#define	EDICT_FROM_AREA(l) STRUCT_FROM_LINK(l,edict_t,area)


//============================================================================

void PR_Init (void);

void PR_ExecuteProgram (func_t fnum);
void PR_ExecuteProgramByName (const char *name);
void PR_ExecuteUIByName(const char *name);

void PR_Profile_f (void);

edict_t *ED_Alloc (void);
void ED_Free (edict_t *ed);

char	*ED_NewString (char *string);
// returns a copy of the string allocated from the server's string heap

void ED_Print (edict_t *ed);
void ED_Write (FILE *f, edict_t *ed);
char *ED_ParseEdict (char *data, edict_t *ent);

void ED_WriteGlobals (FILE *f);
void ED_ParseGlobals (char *data);

void ED_LoadFromFile (char *data);

//define EDICT_NUM(n) ((edict_t *)(sv.edicts+ (n)*pr->edict_size))
//define NUM_FOR_EDICT(e) (((byte *)(e) - sv.edicts)/pr->edict_size)

edict_t *EDICT_NUM(int n);
int NUM_FOR_EDICT(edict_t *e);

#define	NEXT_EDICT(e) ((edict_t *)( (byte *)e + pr->edict_size))

#define	EDICT_TO_PROG(e) ((byte *)e - (byte *)sv.edicts)
#define PROG_TO_EDICT(e) ((edict_t *)((byte *)sv.edicts + e))


#define	G_FLOAT(o) (pr->globals[o])
#define	G_INT(o) (*(int *)&pr->globals[o])
#define	G_EDICT(o) ((edict_t *)((byte *)sv.edicts+ *(int *)&pr->globals[o]))
#define G_EDICTNUM(o) NUM_FOR_EDICT(G_EDICT(o))
#define	G_VECTOR(o) (&pr->globals[o])
#define	G_STRING(o) (pr->strings + *(string_t *)&pr->globals[o])
#define	G_FUNCTION(o) (*(func_t *)&pr->globals[o])

#define	E_FLOAT(e,o) (((float*)&e->v)[o])
#define	E_INT(e,o) (*(int *)&((float*)&e->v)[o])
#define	E_VECTOR(e,o) (&((float*)&e->v)[o])
#define	E_STRING(e,o) (pr->strings + *(string_t *)&((float*)&e->v)[o])


//============================================================================

typedef struct {
	const char *name;
	void (*func) (void);
} builtin_t;

#define MAX_STACK_DEPTH  32

#define LOCALSTACK_SIZE  2048

typedef struct
{
	int  s;
	dfunction_t  *f;

} prstack_t;


#define	MAX_FIELD_LEN	64

#define GEFV_CACHESIZE	2

typedef struct {
	ddef_t	*pcache;
	char	field[MAX_FIELD_LEN];
} gefv_cache_t;


typedef struct
{
	dprograms_t		*progs;

	dfunction_t		* functions;
	char			* strings;
	ddef_t			* fielddefs;
	ddef_t			* globaldefs;
	dstatement_t	* statements;
	globalvars_t	* global_struct;
	float			* globals;			// same as pr->global_struct
	int				  edict_size;	// in bytes

	bool_t  trace;

	int		argc;

	builtin_t * builtins;

	dfunction_t	* xfunction;
	int			  xstatement;

	// call stack
	prstack_t  stack[MAX_STACK_DEPTH];
	int        depth;

	// stack for local variables (including parameters)
	int  localstack[LOCALSTACK_SIZE];
	int  localstack_used;

	gefv_cache_t  gefvCache[GEFV_CACHESIZE];

} prog_vm_t;


void PR_RunError (const char *error, ...);

void ED_PrintEdicts (void);
void ED_PrintNum (int ent);

eval_t *GetEdictFieldValue(edict_t *ed, char *field);

void PR_LoadProgs(prog_vm_t * pr, const char * name, builtin_t * builtins,
                  bool_t crc_check);


dfunction_t *PR_FindFunction (const char *name);


void UI_Init(void);
int  UI_GetMenuState(void);
void UI_SetMouseState(void);


// the game VM
extern prog_vm_t  pr_game;
extern builtin_t  pr_game_builtins[];

// the user interface VM
extern prog_vm_t  pr_ui;
extern builtin_t  pr_ui_builtins[];


// the active VM
extern prog_vm_t *pr;




//--- editor settings ---
// vi:ts=4:sw=4:noexpandtab
