//----------------------------------------------------------------------------
//  Sound Blitter
//----------------------------------------------------------------------------
// 
//  Copyright (c) 2010-2012 Andrew Apted
//  Copyright (c) 1999-2009 The EDGE Team.
//  Copyright (C) 1993-1996 by id Software, Inc.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//----------------------------------------------------------------------------
//
//  Based on the DOOM source code by Id Software
//
//----------------------------------------------------------------------------

#ifndef __S_BLIT__
#define __S_BLIT__


#define NUM_CHANNELS  28


// We use a 22.10 fixed point for sound offsets.  It's a reasonable
// compromise between longest sound and accumulated round-off error.
typedef unsigned long fixed22_t;

typedef enum
{
	CS_EMPTY    = 0,
	CS_PLAYING  = 1,
	CS_FINISHED = 2
}
chan_state_e;


// channel info
typedef struct mix_channel_s
{
	int state;  // CS_xxx

	// points to 'data' in the cache_user_t
	// so we can check if the cache was freed or moved.
	void ** sfx_ptr;

	int category;  // CAT_xxx

	int time;  // how recent the playing sound is
	           // older sounds have lower values

	// TODO: vec3_t *pos;

	fixed22_t offset;
	fixed22_t length;
	fixed22_t delta;

	int volume_L;  // mixing volume
	int volume_R;

	bool_t loop;  // will loop *one* more time

} mix_channel_t;


extern mix_channel_t  mix_chan[NUM_CHANNELS];


void S_InitChannels(void);
void S_FreeChannels(void);

void S_KillChannel(int k);

void S_SetupChannel(int k, void ** sfx_ptr, int category); 

// mix all active channels into the output stream.
// 'len' is the number of samples (for stereo: pairs) to mix.
void S_MixAllChannels(void * udata, byte * stream, int len);

void S_UpdateSounds(vec3_t * listener, float angle);


//-------- API for MUSIC stuffing --------------------

// initialise the queueing system.
void S_Queue_Init(void);

// finalise the queuing system, stopping all playback.
// The data from all the buffers will be freed.
void S_Queue_Shutdown(void);

// returns the next unused (or finished) buffer, or NULL if there
// are none usable at the moment.
//
// The 'length' field is the total number of samples usable in the buffer
// ('channel' will be 1).  The proper rate, channel and interleave fields
// must be set before adding the buffer to the queue.
//
// Only one buffer can be grabbed at a time.
sound_t * S_Queue_GetFreeBuffer(void);

// mark the buffer gotten from S_Queue_GetFreeBuffer() to be playable
// (conceptually it has been added to the end of the queue).
void S_Queue_AddBuffer(void);

#endif // __S_BLIT__

//--- editor settings ---
// vi:ts=4:sw=4:noexpandtab
