
#include <stdio.h>

#include <SDL.h>
#include <SDL_audio.h>
#include <SDL_byteorder.h>

#include "main.h"
#include "snd_blit.h"


sounddef_t  snd;


cvar_t volume = {"volume", "0.5", true};
cvar_t mus_volume = {"mus_volume", "0.5", true};

cvar_t ambient_level = {"ambient_level", "0.3"};
cvar_t ambient_fade = {"ambient_fade", "100"};
cvar_t snd_noextraupdate = {"snd_noextraupdate", "0"};
cvar_t snd_show = {"snd_show", "0"};



bool_t SNDDMA_Init(void)
{
	SDL_AudioSpec desired, obtained;

	snd.disabled = true;

	/* Set up the desired format */
	desired.freq = 22050;
	desired.channels = 2;
	desired.format = AUDIO_S16SYS;
	desired.samples = 512;
	desired.callback = S_MixAllChannels;

#if 0
	switch (16)
	{
		case 8:
			desired.format = AUDIO_U8;
			break;
		case 16:
			desired.format = AUDIO_S16SYS;
			break;

		default:
       		Con_Printf("Unknown number of audio bits: %d\n", desired_bits);
			return 0;
	}
#endif

	/* Open the audio device */
	if ( SDL_OpenAudio(&desired, &obtained) < 0 )
	{
       	Con_Printf("Couldn't open SDL audio: %s\n", SDL_GetError());
		return 0;
	}

	/* Make sure we can support the audio format */
	switch (obtained.format)
	{
		case AUDIO_U8:
		case AUDIO_S8:
			/* Supported */
			break;

		case AUDIO_S16SYS:
		case AUDIO_U16SYS:
				/* Supported */
				break;

		default:
			/* Not supported -- force SDL to do our bidding */
			SDL_CloseAudio();

			if ( SDL_OpenAudio(&desired, NULL) < 0 )
			{
       			Con_Printf("Couldn't open SDL audio: %s\n", SDL_GetError());
				return 0;
			}
			memcpy(&obtained, &desired, sizeof(desired));
			break;
	}

	/* Fill the audio information block */

	snd.rate     = obtained.freq;
	snd.channels = obtained.channels;
	snd.bits     = (obtained.format & 0xFF);

	snd.is_signed = (obtained.format == AUDIO_S8     ||
	                 obtained.format == AUDIO_S16SYS ||
	                 obtained.format == AUDIO_S16LSB ||
	                 obtained.format == AUDIO_S16MSB);

	snd.sample_size = snd.channels * (snd.bits / 8);
	snd.frag_pairs  = obtained.size / snd.sample_size;

	snd.disabled = false;

	
	S_InitChannels();


	SDL_PauseAudio(0);

	Con_Printf("Sound Info: %d Hz, %d bits, %d channels, %d frags\n",
			   snd.rate, snd.bits, snd.channels, snd.frag_pairs);

	return 1;
}


void S_Shutdown(void)
{
	if (! snd.disabled)
	{
		SDL_CloseAudio();
		snd.disabled = true;
	}
}


void S_Init (void)
{
	Con_Printf("\nSound Initialization\n");

	Cvar_RegisterVariable(&volume);
	Cvar_RegisterVariable(&mus_volume);
	Cvar_RegisterVariable(&ambient_level);
	Cvar_RegisterVariable(&ambient_fade);
	Cvar_RegisterVariable(&snd_noextraupdate);
	Cvar_RegisterVariable(&snd_show);

	if (COM_CheckParm("-nosound"))
		return;

	if (! SNDDMA_Init())
		return;

	S_InitSoundTable();

	S_Queue_Init();
	S_OGG_Init();

#if 0  // FIXME
	Cmd_AddCommand("play", S_Play);
	Cmd_AddCommand("playvol", S_PlayVol);
	Cmd_AddCommand("stopsound", S_StopAllSoundsC);
	Cmd_AddCommand("soundlist", S_SoundList);
	Cmd_AddCommand("soundinfo", S_SoundInfo_f);
#endif

}


void S_Lock(void)
{
	if (! snd.disabled)
		SDL_LockAudio();
}


void S_Unlock(void)
{
	if (! snd.disabled)
		SDL_UnlockAudio();
}


//----------------------------------------------------------------------


sound_t * SND_Create(int length, int rate, int channels, struct cache_user_s * cu)
{
	unsigned int size;

	sound_t * sfx;


	size = sizeof(sound_t);
	
	size += length * channels * sizeof(short);


	if (cu)
		sfx = (sound_t *) Cache_Alloc(cu, size, "sound");
	else
		sfx = (sound_t *) Hunk_AllocName(size, "sound");
	
	if (! sfx)
		return NULL;
	

	sfx->length   = length;
	sfx->rate     = rate;
	sfx->channels = channels;
	sfx->interleaved = false;

	return sfx;
}


//--- editor settings ---
// vi:ts=4:sw=4:noexpandtab
