/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// sound.h -- client sound i/o functions

#ifndef __SOUND__
#define __SOUND__

#define DEFAULT_SOUND_PACKET_VOLUME 255
#define DEFAULT_SOUND_PACKET_ATTENUATION 1.0


typedef struct
{
	bool_t	disabled;		// no sounds for you, buddy

	int		rate;			// device frequency (e.g. 22050)
	int		channels;		// device # of channels (1 or 2)
	int		bits;			// device bit size (8 or 16)
	bool_t	is_signed;		// samples are signed (rather than unsigned)

	int		sample_size;	// size of each sample in bytes (1 to 4)
	int		frag_pairs;		// suggested buffer (fragment) size

} sounddef_t;

extern sounddef_t  snd;  // global sound info


// Sound Categories
// ----------------
//
// Each category has a minimum number of channels (say N), and
// sounds from one category never cut off sounds from another one.
// When a category is full and a sound is played on it, the longest
// playing sound gets cut off.
//
typedef enum
{
	CAT_UI = 0,     // for the user interface (menus, tips)   [4]
	CAT_PLAYER,     // for all players (pain, death, pickup)  [8]
	CAT_MONSTER,    // for all monster sounds                 [8]
	CAT_LEVEL,      // for all other entities, doors, etc     [4]
	CAT_AMBIENT,    // for ambient (background) sounds        [4]

	CAT_TOTAL_NUM
}
sound_category_e;


typedef struct sound_s
{
	int		length;    // number of samples
	int		rate;      // frequency (Hz)
	int		channels;  // 1 or 2
	bool_t	interleaved;

	short	samples[1];  // variable length

} sound_t;


extern cvar_t  volume;
extern cvar_t  mus_volume;


void S_Init (void);
void S_Shutdown (void);

void S_StartFX(int sound_num, int category, float vol);

void S_StopSound (int entnum, int entchannel);
void S_StopAllSounds(bool_t clear);

void S_Update (vec3_t origin, vec3_t v_forward, vec3_t v_right, vec3_t v_up);
void S_ExtraUpdate (void);

void S_Lock(void);
void S_Unlock(void);


void S_InitSoundTable (void);

// returns 'sound_num' : an index into the sound table
int S_FindSound (const char * name);

void S_CacheSound(int tab_idx);


sound_t * SND_Create(int length, int rate, int channels,
                     struct cache_user_s * cu);

// cache a WAV sound into memory (from file or pack)
// normally loads into a cache, but if 'cu' is NULL then loaded on hunk
// returns NULL if no such file / couldn't parse file
sound_t * S_CacheWAV(const char * name, struct cache_user_s * cu);


void	S_OGG_Init(void);
bool_t	S_OGG_Open(const char *name);
void	S_OGG_Close(void);
void	S_OGG_Update(void);


#endif

//--- editor settings ---
// vi:ts=4:sw=4:noexpandtab
