//------------------------------------------------------------------------
//  Image Handling
//------------------------------------------------------------------------
//
//  Copyright (c) 2006-2012  Andrew Apted
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//------------------------------------------------------------------------
//
//  Based on IMG_tga.c in the SDL_image library.
//  SDL_image is Copyright (C) 1997-2006 Sam Lantinga.
//
//------------------------------------------------------------------------

#include "main.h"


image_t * IMG_Create(int width, int height, struct cache_user_s *cu)
{
	// allocate the image_t + pixel data following it

	unsigned int size = sizeof(image_t) + (width * height * sizeof(rgbcol_t));

	image_t * img;
	
	if (cu)
		img = (image_t *) Cache_Alloc(cu, size, "image");
	else
		img = (image_t *) Hunk_AllocName(size, "image");

	if (! img)
		return NULL;

	img->width    = width;
	img->height   = height;
	img->is_solid = true;

	return img;
}


static inline void SwapRows(image_t * img, int y1, int y2)
{
	if (y1 == y2)
		return;
	
	rgbcol_t * p1 = img->pixels + y1 * img->width;
	rgbcol_t * p2 = img->pixels + y2 * img->width;

	rgbcol_t * p1_end = p1 + img->width;

	for ( ; p1 < p1_end ; p1++, p2++)
	{
		rgbcol_t tmp = *p1;  *p1 = *p2;  *p2 = tmp;
	}
}


void IMG_Invert(image_t * img)
{
	int y;

	for (y = 0 ; y < img->height/2 ; y++)
	{
		SwapRows(img, y, img->height - 1 - y);
	}
}


//--- editor settings ---
// vi:ts=4:sw=4:noexpandtab
