//------------------------------------------------------------------------
//  Image Handling
//------------------------------------------------------------------------
//
//  Copyright (c) 2006-2012  Andrew Apted
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//------------------------------------------------------------------------

#ifndef ___V_IMAGE_H___
#define ___V_IMAGE_H___

typedef unsigned int rgbcol_t;

#define ALPHA_SOLID  255
#define ALPHA_TRANS    0


#define MAKE_RGB(r,g,b)     (((r) << 16) | ((g) << 8) | (b) | (ALPHA_SOLID << 24))
#define MAKE_RGBA(r,g,b,a)  (((r) << 16) | ((g) << 8) | (b) | ((a)<<24))

#define RGB_R(col)  (((col) >> 16) & 0xFF)
#define RGB_G(col)  (((col) >>  8) & 0xFF)
#define RGB_B(col)  (((col)      ) & 0xFF)
#define RGB_A(col)  (((col) >> 24) & 0xFF)


typedef struct image_s
{
	int width;
	int height;

	int is_solid;  // true when no transparent parts exist

	rgbcol_t pixels[1];  // variable sized

} image_t;


// images are created on the hunk (hence no IMG_Free)
// normally created into a cache, but if 'cu' is NULL then sits on hunk
// returns NULL if not enough memory
image_t * IMG_Create(int width, int height, struct cache_user_s *cu);

// invert the image (turn it up-side-down)
void IMG_Invert(image_t * img);

// cache a TGA format image (from file or pack)
// normally loads into a cache, but if 'cu' is NULL then loaded on hunk
// returns NULL if no such file / couldn't parse file
image_t * V_CacheTGA(const char * name, struct cache_user_s * cu);


#endif  /* ___V_IMAGE_H___ */

//--- editor settings ---
// vi:ts=4:sw=4:noexpandtab
